/*
	XMMS Song Info plugin
	Copyright (c) 2004 Nikola Kotur

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/

#include <string.h>
#include <stdio.h>
#include <id3tag.h>
#include "mp3info.h"

/* id3_get_tag taken from mpg321 sourcecode */

/* Convenience for retrieving already formatted id3 data
 * what parameter is one of
 *  ID3_FRAME_TITLE
 *  ID3_FRAME_ARTIST
 *  ID3_FRAME_ALBUM
 *  ID3_FRAME_YEAR
 *  ID3_FRAME_COMMENT
 *  ID3_FRAME_GENRE
 * It allocates a new string. Free it later.
 * NULL if no tag or error.
 */
char *id3_get_tag (struct id3_tag const *tag, char const *what, unsigned int maxlen)
{
	struct id3_frame const *frame = NULL;
	union id3_field const *field = NULL;
	int nstrings;
	int avail;
	int j;
	int tocopy;
	int len;
	char printable [1024];
	char *retval = NULL;
	id3_ucs4_t const *ucs4 = NULL;
	id3_latin1_t *latin1 = NULL;

	memset (printable, '\0', 1024);
	avail = 1024;
	if (strcmp (what, ID3_FRAME_COMMENT) == 0)
	{
		/*There may be sth wrong. I did not fully understand how to use
		libid3tag for retrieving comments  */
		j=0;
		frame = id3_tag_findframe(tag, ID3_FRAME_COMMENT, j++);
		if (!frame) return (NULL);
		ucs4 = id3_field_getfullstring (&frame->fields[3]);
		if (!ucs4) return (NULL);
		latin1 = id3_ucs4_latin1duplicate (ucs4);
		if (!latin1 || strlen((const char*)latin1) == 0) return (NULL);
		len = strlen((const char*)latin1);
		if (avail > len)
			tocopy = len;
		else
			tocopy = 0;
		if (!tocopy) return (NULL);
		avail-=tocopy;
		strncat (printable, (const char*)latin1, tocopy);
		free (latin1);
	}
	else
	{
		frame = id3_tag_findframe (tag, what, 0);
		if (!frame) return (NULL);
		field = &frame->fields[1];
		nstrings = id3_field_getnstrings(field);
		for (j=0; j<nstrings; ++j)
		{
			ucs4 = id3_field_getstrings(field, j);
			if (!ucs4) return (NULL);
			if (strcmp (what, ID3_FRAME_GENRE) == 0)
				ucs4 = id3_genre_name(ucs4);
			latin1 = id3_ucs4_latin1duplicate(ucs4);
			if (!latin1) break;
			len = strlen((const char*)latin1);
			if (avail > len)
				tocopy = len;
			else
				tocopy = 0;
			if (!tocopy) break;
			avail-=tocopy;
			strncat (printable, (const char*)latin1, tocopy);
			free (latin1);
		}
	}
	retval = (char*)calloc (maxlen + 1, 1);
	if (!retval) return (NULL);

	strncpy (retval, printable, maxlen);
	retval[maxlen] = '\0';

	len = strlen(printable);
	if (maxlen > len)
	{
		memset (retval + len, '\0', maxlen - len);
	}

	return (retval);
}

#define get_field(tag, field, target, len) \
	pfield = id3_get_tag(tag, field, len); \
	if (pfield) { \
		strncpy(target, pfield, len); \
		free(pfield); \
	}


int getmp3info(char *fname, ext_id3_record *rec)
{
	rec->is_valid = 0;

	struct id3_file *id3struct = id3_file_open(fname, ID3_FILE_MODE_READONLY);

	if (id3struct) {
		struct id3_tag *id3tag = id3_file_tag(id3struct);

		if (id3tag && id3tag->nframes > 0) {
			char *pfield;

			get_field(id3tag, ID3_FRAME_ARTIST, rec->artist, EXT_ID3_FLEN);
			get_field(id3tag, ID3_FRAME_TITLE, rec->title, EXT_ID3_FLEN);
			get_field(id3tag, ID3_FRAME_ALBUM, rec->album, EXT_ID3_FLEN);
			get_field(id3tag, ID3_FRAME_COMMENT, rec->comment, EXT_ID3_FLEN);
			get_field(id3tag, ID3_FRAME_YEAR, rec->year, EXT_ID3_FLEN);

			rec->is_valid = 1;
		}

		id3_file_close (id3struct);
		return 1;
	}
	return 0;
}

